import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  curveBasis,
  line,
  area,
} from 'd3';

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

const render = (data) => {
  const xValue = (d) => new Date(d.Date);
  const xAxisLabel = 'Date';

  const yValue = (d) => +d['Average Temperature (°C)'];
  const yAxisLabel = 'Average Temperature (°C)';

  const margin = {
    top: 40,
    right: 40,
    bottom: 60,
    left: 60,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(10);

  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -45)
    .attr('x', -innerHeight / 2)
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 40)
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis);

  const areaGenerator = area()
    .x((d) => xScale(xValue(d)))
    .y0(innerHeight)
    .y1((d) => yScale(yValue(d)))
    .curve(curveBasis);

  g.append('path')
    .attr('class', 'area-chart')
    .attr('d', areaGenerator(data));

  g.append('path')
    .attr('class', 'line-chart')
    .attr('d', lineGenerator(data));
};

csv(
  'https://gist.githubusercontent.com/ArchanaGoli/a612028a1c06123b8ec03ffa58089631/raw/01855a581adcb624ac28fd3c63817e0aa7af858b/gistfile1.txt',
).then((data) => {
  data.forEach((d) => {
    d['Average Temperature (°C)'] =
      +d['Average Temperature (°C)'];
    d.Date = new Date(d.Date);
  });
  render(data);
